/* $USAGI: ifaddrs_test.c,v 1.9 2005/06/06 08:33:36 yoshfuji Exp $ */

/*
 * Copyright (C) 2001 USAGI/WIDE Project.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#ifdef HAVE_IFADDRS_H
#include <ifaddrs.h>
#endif
#include <netdb.h>
#include <netpacket/packet.h>
#include <net/if_arp.h>

#ifndef HAVE_GETIFADDRS
#error "you don't have getifaddrs()!"
#endif

#ifndef HAVE_FUNC_SA_LEN
#define SA_LEN(sa)	__ac_salen(sa->sa_family)
static inline	socklen_t __ac_salen(sa_family_t family)
{
	switch(family) {
	case AF_INET:	return sizeof(struct sockaddr_in);
#if defined(AF_INET6) && defined(HAVE_STRUCT_SOCKADDR_IN6)
	case AF_INET6:	return sizeof(struct sockaddr_in6);
#endif
	}
	return 0;
}
#endif

#define PRINT_IFAFLAGS(flags,flag) ({				\
	if (flags & IFF_ ## flag) {				\
		printf(" %s", # flag);				\
		flags &= ~(IFF_ ## flag);			\
	}							\
})								\

void print_ifaflags(unsigned flags)
{
	int count = 0;
	PRINT_IFAFLAGS(flags, LOOPBACK);
	PRINT_IFAFLAGS(flags, POINTOPOINT);
	PRINT_IFAFLAGS(flags, MULTICAST);
	PRINT_IFAFLAGS(flags, BROADCAST);
	PRINT_IFAFLAGS(flags, PROMISC);
	PRINT_IFAFLAGS(flags, ALLMULTI);
	PRINT_IFAFLAGS(flags, NOARP);
	PRINT_IFAFLAGS(flags, UP);
	PRINT_IFAFLAGS(flags, RUNNING);
	if (flags)
		printf(" unknown(%08x)", flags);
	printf("\n");	
}

void print_sa(const char *title, struct sockaddr *sa) {
	int gni;
	char buf[128];
	char *p = buf;
	if (sa) {
		switch(sa->sa_family) {
		case AF_INET:
		case AF_INET6:
			p += sprintf(buf, "%s addr = ", 
					sa->sa_family == AF_INET6 ?
					"ipv6" : "ipv4");
			gni = getnameinfo(sa, SA_LEN(sa), 
					  p, sizeof(buf)-strlen(buf)-1, NULL, 0,
					  NI_NUMERICHOST);
			if (gni)
				strcpy(buf, "<unknown>");
			break;
		case AF_PACKET:
		    {
			struct sockaddr_ll *sll = (struct sockaddr_ll *)sa;
			char *p = buf;
			int i;
			char *hatype = NULL;
			switch(sll->sll_hatype){
			case ARPHRD_LOOPBACK:
				hatype = "loopback";
				break;
			case ARPHRD_ETHER:
				hatype = "Ethernet";
				break;
			case ARPHRD_IEEE802:
				hatype = "IEEE802";
				break;
			case ARPHRD_ARCNET:
				hatype = "ARCnet";
				break;
			case ARPHRD_TUNNEL:
				hatype = "IPIP tunnel";
				break;
			case ARPHRD_TUNNEL6:
				hatype = "IPIP6 tunnel";
				break;
			case ARPHRD_SIT:
				hatype = "SIT tunnel";
				break;
			case ARPHRD_IPGRE:
				hatype = "GRE tunnel";
				break;
			case ARPHRD_FDDI:
				hatype = "FDDI";
				break;
			}
			if (hatype) {
				p += sprintf(p, "%s", hatype);
			} else {
				p += sprintf(p, "type %d", sll->sll_hatype);
			}
			p += sprintf(p, " addr = ");
			for (i=0; i<sll->sll_halen; i++){
				p += sprintf(p, "%s%02x", i?":":"",
						sll->sll_addr[i]);
			}
		    }
			break;
		default:
			sprintf(buf, "unknown family %d", sa->sa_family);
		}
	} else {
		strcpy(buf, "<null>");
	}
	printf("\t%-14s: %s\n", title, buf);
}

int main(int argc, char **argv){
	struct ifaddrs *ifa0, *ifa;
	int count = 0;
	char *iface = NULL;

	if (argc > 2) {
		fprintf(stderr, "Usage: %s [ifname]\n", argv[0]);
		exit(1);
	} else if (argc == 2) {
		iface = argv[1];
	}

	if (getifaddrs(&ifa0)){
		perror("getifaddrs()");
		exit(4);
	}
	if (!ifa0) {
		fprintf(stderr, "Error: interface not found?!\n");
		exit(2);
	}

	for (ifa=ifa0; ifa; ifa=ifa->ifa_next) {
		if (iface && ifa->ifa_name && strcmp(ifa->ifa_name, iface))
			continue;
		count++;
		printf("%d: %s, flag=%04x\n", 
			count, 
			ifa->ifa_name ? ifa->ifa_name : "<null>",
			ifa->ifa_flags
		);
		if (!ifa->ifa_addr) {
			printf("\tifa_addr=NULL\n");
			continue;
		}
		print_sa("ifa_addr", ifa->ifa_addr);
		print_sa("ifa_netmask", ifa->ifa_netmask);
		print_sa("ifa_broadaddr", ifa->ifa_broadaddr);
		printf("\t%-14s: 0x%08x:", "ifa_flags", ifa->ifa_flags);
		print_ifaflags(ifa->ifa_flags);
	}

	if (ifa0 && !count) {
		fprintf(stderr, "No such interface.\n");
		exit(1);
	}

	exit(0);
}
